/*    
 *    Copyright (c) 2010 LongTail AdSolutions, Inc
 *
 *    This file is part of the Open Video Ads VAST framework.
 *
 *    The VAST framework is free software: you can redistribute it 
 *    and/or modify it under the terms of the GNU General Public License 
 *    as published by the Free Software Foundation, either version 3 of 
 *    the License, or (at your option) any later version.
 *
 *    The VAST framework is distributed in the hope that it will be 
 *    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with the framework.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    @author  Paul Schulz
 *    @date    March 20, 2012
 *    @contact enquiries@openvideoads.org (www.openvideoads.org)
 * 
 */

function NativeOVAImplementation(_player, _config) {

 	// Analytics Constants
	this.OVA_JW5_DEFAULT_LINEAR_IMPRESSION_PATH = "/ova/impression/jw5-html?ova_format=linear";
	this.OVA_JW5_DEFAULT_NON_LINEAR_IMPRESSION_PATH = "/ova/impression/jw5-html?ova_format=non-linear";
	this.OVA_JW5_DEFAULT_COMPANION_IMPRESSION_PATH = "/ova/impression/jw5-html?ova_format=companion";
	this.OVA_JW5_DEFAULT_GA_ACCOUNT_ID = "UA-4011032-6";
	this.GOOGLE_ANALYTICS = "GA";
	this.OVA = "OVA";
    
    /*
     * Start the native implementation running
     */
    this.start = function() {
		if(eval("typeof vastController == 'object'") == false) {
		    $.getScript('http://localhost/ova/ova.html5/lib/ova-html5.js',
		        function() {
	 	           _initialise(_config);
		        }
		    );
		}
	 	else _initialise(_config);
	}

    /*
     * Initialise the plugin
     */
	this.initialise = function(config) {
		_initialise(config, predefinedPlaylist, overrideDelayAdRequestConfig);
	}

    /*
     * Method
     */
    function _validateConfig(config) {
		var _ovaConfig = null;
		
		if(config.json == null && (config.tag != null || config.vast != null)) {
			// Minimal setup of a VAST ad tag via the "tag" variable
			_ovaConfig = {
			    "ads": {
                    "companions": {
                    	"restore": false,
                     	"regions": [
                            { "id": "companion-300x250", "width": 300, "height": 250 }
                        ]
                    },
                    "schedule": [
                        {
                            "position": "pre-roll",
                            "server": {
                                "type": ((config.tag != null) ? "direct" : "inject"),
	                            "tag": ((config.tag != null) ? config.tag : config.vast)
	                        }
                        }
                    ]
			    }
			};
			if(config.debug != null) {
				_ovaConfig.debug = { "levels": config.debug };
			}
			else _ovaConfig.debug = { "levels": "fatal, config, vast_template, vpaid" };
			if(config.autoplay != null) _ovaConfig.autoPlay =  config.autoplay;
			config.tag = null;
		}
		else if(config.json != null) {
			try {
				_ovaConfig = jQuery.parseJSON(config.json)
			}
			catch(e) {
				ovaDebug.out("OVA Configuration parsing exception - " + e.description, ovaDebug.DEBUG_CONFIG);
				return null;
			}
			if(config.tag != null) {
				if(_ovaConfig == null) {
					_ovaConfig = { tag: config.tag };
				}
				else _ovaConfig.tag = config.tag;
			}
		}
		else if(config.ads != undefined || config.debug != undefined) {
			_ovaConfig = config;
		}

		if(_ovaConfig != null) {
			// now set any defaults if not configured manually within OVA config

			if(_ovaConfig.debug == undefined) _ovaConfig.debug = { "levels": "fatal, config" };
			ovaDebug.initialise(_ovaConfig.debug);
			if(_ovaConfig.autoPlay == undefined) {
				_ovaConfig.autoPlay = false;
			}
			else if(typeof _ovaConfig.autoPlay == "string") {
				_ovaConfig.autoPlay = new Boolean(_ovaConfig.autoPlay);
			}
			if(_player.config.autostart) {
				_ovaConfig.autoPlay = true;
				_player.config.autostart = (_player.config.repeat == "always");
				ovaDebug.out("autoPlay set to true as 'autostart' flashvar specified - endless looping = " + _player.config.autostart, ovaDebug.DEBUG_CONFIG);
			}
			if(config.json != undefined) {
				ovaDebug.out("Config has been loaded via a 'json' string (dump follows) - " + config.json, ovaDebug.DEBUG_CONFIG);
			}
			else {
				ovaDebug.out("Have attached the config object passed from player (dump follows)", ovaDebug.DEBUG_CONFIG);
			}
			ovaDebug.out(_ovaConfig, ovaDebug.DEBUG_CONFIG);
		}
		else {
			_ovaConfig = {
				json: null,
				tag: null,
				debug: null
			};
			ovaDebug.initialise({ "levels": "fatal, config" });
			ovaDebug.out("No OVA configuration provided - ad streamer will not be capable of playing any ads", ovaDebug.DEBUG_CONFIG);
			return null;
		}

		if(config.tagparams != undefined && config.tagparams != null) {
			// tag properties allow the "customProperties" element of an ad server configuration to be passed through
			_ovaConfig.tagParams = config.tagparams.paramStringToObject(",", ":");
			ovaDebug.out("Additional Ad Tag Parameters have been specified - they will be added to the ad request(s)", ovaDebug.DEBUG_CONFIG);
		}

		// TO DO: vastController.additionMetricsParams = "ova_plugin_version=" + OVA_VERSION + "&ova_player_version=" + _player.version;

		if(_ovaConfig.ads != undefined) {
			// set the default mime types allowed - can be overridden in config with "acceptedLinearAdMimeTypes" and "filterOnLinearAdMimeTypes"
			if(_ovaConfig.ads.filterOnLinearAdMimeTypes == undefined) {
				ovaDebug.out("Setting accepted Linear Ad mime types to default list - mp4, webm");
				_ovaConfig.ads.acceptedLinearAdMimeTypes = [ "video/mp4", "video/x-mp4", "mp4", "video/webm", "video/x-webm", "webm" ];
				_ovaConfig.ads.filterOnLinearAdMimeTypes = true;
			}
			else ovaDebug.out("Setting accepted Linear Ad mime types and enabled state = " + _ovaConfig.ads.filterOnLinearAdMimeTypes, ovaDebug.DEBUG_CONFIG);
		}  
		
		return _ovaConfig;  	
    }	

    /*
     * Method
     */
    function _initialise(config) {
        ovaDebug.out("Initialising the native OVA for JW5 HTML5 implementation", ovaDebug.DEBUG_CONFIG);
		var _ovaConfig = _validateConfig(config);
		if(_ovaConfig != null) {
		    // Add the playlist to the OVA "shows" configuration block so that it can be used in Ad Scheduling
			_addExistingPlaylistToOVAConfig(_ovaConfig);
	
	        // These are needed if the ad server call requires the player width and height to be passed through
	        vastController.setPlayerConfig(
	        		{ 
	        			width: _getPlayerWidth(),
	        			height: _getPlayerHeight(),
	        			startStreamSafetyMargin: 300,
	        			endStreamSafetyMargin: 300,
	        			volume: _player.getVolume()
	        		}
	        );
	        
		    // Initialise the controller given the OVA configuration that has been provided
		    vastController.initialise(_ovaConfig);

			// Register the two sets of handlers
			_registerPlayerHandlers();
			_registerOVAHandlers();

			// Setup the Google Analytics impression counter
	        vastController.registerAnalytics(
	        		[
	        			{
	        				type: this.GOOGLE_ANALYTICS,
	        				element: this.OVA,
	        				displayObject: this,
	            			analyticsId: this.OVA_JW5_DEFAULT_GA_ACCOUNT_ID,
	            			impressions: {
	            				linear: this.OVA_JW5_DEFAULT_LINEAR_IMPRESSION_PATH,
		        				nonLinear: this.OVA_JW5_DEFAULT_NON_LINEAR_IMPRESSION_PATH,
	    	    				companion: this.OVA_JW5_DEFAULT_COMPANION_IMPRESSION_PATH
	    	    			}
	        			}
	        		]
	        );

			ovaDebug.out("OVA initialisation complete. Triggering ad scheduling...", ovaDebug.DEBUG_ALL);

    	    // Ok, let's load up the VAST data from our Ad Server
        	vastController.load();
		}
		else _abort();
	}

    /*
     * Method
     */
    function _abort() {
    	// TO DO
	}

    /*
     * Method
     */
    function _registerPlayerHandlers() {
		_player.onPlaylist(_onPlaylistLoadedAtStartupEvent);
 	    _player.onTime(_onTimeEvent);
	    _player.onMeta(_onMetaDataEvent);
	    _player.onMute(_onMuteEvent);
	    _player.onVolume(_onVolumeChangeEvent);
	    _player.onPlay(_onPlayEvent);
	    _player.onIdle(_onIdleEvent); 
	    _player.onComplete(_onCompleteEvent);
	    _player.onPause(_onPauseEvent);
	    _player.onBuffer(_onBufferEvent);
	    _player.onFullscreen(_onFullscreenEvent);
	    _player.onError(_onErrorEvent);
	}

    /*
     * Method
     */
    function _registerOVAHandlers() {
        // Setup the critical listeners for the template loading process - used by the ad slot "preloaded model"
	    vastController.addEventListener(TemplateEvent.LOADED, this.onTemplateLoaded);
	    vastController.addEventListener(TemplateEvent.LOAD_FAILED, this.onTemplateLoadError);
	    vastController.addEventListener(TemplateEvent.LOAD_TIMEOUT, this.onTemplateLoadTimeout);
	    vastController.addEventListener(TemplateEvent.LOAD_DEFERRED, this.onTemplateLoadDeferred);

        // Setup the critical listeners for the ad slot loading process - used by the ad slot "on demand load model"
	    vastController.addEventListener(AdSlotLoadEvent.LOADED, this.onAdSlotLoaded);
	    vastController.addEventListener(AdSlotLoadEvent.LOAD_ERROR, this.onAdSlotLoadError);
	    vastController.addEventListener(AdSlotLoadEvent.LOAD_TIMEOUT, this.onAdSlotLoadTimeout);
	    vastController.addEventListener(AdSlotLoadEvent.LOAD_DEFERRED, this.onAdSlotLoadDeferred);

        // Setup the companion display listeners
	    vastController.addEventListener(CompanionAdDisplayEvent.DISPLAY, this.onDisplayCompanionAd);
	    vastController.addEventListener(CompanionAdDisplayEvent.HIDE, this.onHideCompanionAd);

        // Setup standard overlay event handlers
        vastController.addEventListener(OverlayAdDisplayEvent.DISPLAY, _onDisplayOverlay);
        vastController.addEventListener(OverlayAdDisplayEvent.HIDE, _onHideOverlay);
        vastController.addEventListener(OverlayAdDisplayEvent.DISPLAY_NON_OVERLAY, _onDisplayNonOverlay);
        vastController.addEventListener(OverlayAdDisplayEvent.HIDE_NON_OVERLAY, _onHideNonOverlay);
        vastController.addEventListener(OverlayAdDisplayEvent.CLICKED, _onOverlayClicked);
        vastController.addEventListener(OverlayAdDisplayEvent.CLOSE_CLICKED, _onOverlayCloseClicked);

        // Setup ad notice event handlers
        vastController.addEventListener(AdNoticeDisplayEvent.DISPLAY, _onDisplayNotice);
        vastController.addEventListener(AdNoticeDisplayEvent.HIDE, _onHideNotice);

        // Setup linear tracking events
        vastController.addEventListener(LinearAdDisplayEvent.SKIPPED, _onLinearAdSkipped);
        vastController.addEventListener(LinearAdDisplayEvent.CLICK_THROUGH, _onLinearAdClickThrough);

        // Setup the hander for tracking point set events
        vastController.addEventListener(TrackingPointEvent.SET, _onSetTrackingPoint);
        vastController.addEventListener(TrackingPointEvent.FIRED, _onTrackingPointFired);

        // Setup the hander for display events on the seeker bar
        vastController.addEventListener(OVAControlBarEvent.ENABLE, _onControlBarEnable);
        vastController.addEventListener(OVAControlBarEvent.DISABLE, _onControlBarDisable);
	}


    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    // PLAYER DISPLAY METHODS
    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

    /*
     * Method
     */
    function _getPlayerWidth() {
		if(vastController != null) {
			if(vastController.initialised) {
				if((_player.fullscreen == false) && vastController.config.hasPlayerWidth()) {
					return vastController.config.playerWidth;
				}
			}
		}
		if(_player != null) {
			return _player.getWidth();
		}
		return -1;
    }


    /*
     * Method
     */
    function _getPlayerHeight() {
		if(vastController != null) {
			if(vastController.initialised) {
				if((_player.fullscreen == false) && vastController.config.hasPlayerHeight()) {
					return vastController.config.playerHeight;
				}
			}
		}
		if(_player != null) {
			return _player.getHeight();
		}
		return -1;
    }


    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    // PLAYLIST RELATED METHODS
    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

    /*
     * Method
     */
    function _addExistingPlaylistToOVAConfig(_ovaConfig) {
	}
	
    /*
     * Method
     */	
	function _onPlaylistLoadedAtStartupEvent(event) {
		
	}

    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    // PLAYER EVENT HANDLERS
    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

    /*
     * Method
     */
    function _onTimeEvent(event) {
        /*
	    TO DO
        if(vastController != null) {
           vastController.processTimeEvent(new TimeEvent(event.position))
        }
        */
    }

    /*
     * Method
     */
    function _onMetaDataEvent(event) {
	    // TO DO
        ovaDebug.out("on meta data event", ovaDebug.DEBUG_TRACKING_EVENTS);
    }

    /*
     * Method
     */
	function _onMuteEvent(event) {
	    // TO DO
        ovaDebug.out("on mute event", ovaDebug.DEBUG_TRACKING_EVENTS);
	}

    /*
     * Method
     */
	function _onVolumeChangeEvent(event) {
	    // TO DO
        ovaDebug.out("on volume change event", ovaDebug.DEBUG_TRACKING_EVENTS);
	}

    /*
     * Method
     */
	function _onPlayEvent(event) {
	    // TO DO
        ovaDebug.out("on play event", ovaDebug.DEBUG_TRACKING_EVENTS);
	}

    /*
     * Method
     */
	function _onIdleEvent(event) {
	    // TO DO
        ovaDebug.out("on idle event", ovaDebug.DEBUG_TRACKING_EVENTS);
	}

    /*
     * Method
     */
	function _onCompleteEvent(event) {
	    // TO DO
        ovaDebug.out("on complete event", ovaDebug.DEBUG_TRACKING_EVENTS);
	}

    /*
     * Method
     */
	function _onPauseEvent(event) {
	    // TO DO
        ovaDebug.out("on pause event", ovaDebug.DEBUG_TRACKING_EVENTS);
	}

    /*
     * Method
     */
	function _onBufferEvent(event) {
	    // TO DO
        ovaDebug.out("on buffer event", ovaDebug.DEBUG_TRACKING_EVENTS);
	}

    /*
     * Method
     */
	function _onFullscreenEvent(event) {
	    // TO DO
        ovaDebug.out("on fullscreen event", ovaDebug.DEBUG_TRACKING_EVENTS);
	}

    /*
     * Method
     */
	function _onErrorEvent(event) {
	    // TO DO
        ovaDebug.out("on error event", ovaDebug.DEBUG_TRACKING_EVENTS);
	}


    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    // VAST LOAD CALLBACKS
    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

    /*
     * Method
     */
    function _onTemplateLoaded() {
	    // TO DO
    }

    /*
     * Method
     */
    function _onTemplateLoadError() {
	    // TO DO
    }

    /*
     * Method
     */
    function _onTemplateLoadTimeout() {
	    // TO DO
    }

    /*
     * Method
     */
    function _onTemplateLoadDeferred() {
	    // TO DO
    }

    // ON-DEMAND AD SLOT LOADING

    /*
     * Method
     */
    function _onAdSlotLoadedfunction() {
	    // TO DO
    }

    /*
     * Method
     */
    function _onAdSlotLoadErrorfunction() {
	    // TO DO
    }

    /*
     * Method
     */
    function _onAdSlotLoadTimeoutfunction() {
	    // TO DO
    }

    /*
     * Method
     */
    function _onAdSlotLoadDeferred() {
	    // TO DO
    }

    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    // AD NOTICE DISPLAY
    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

    /*
     * Method
     */
	function _onDisplayNotice(event) {
	    // TO DO
	}

    /*
     * Method
     */
	function _onHideNotice(event) {
	    // TO DO
	}

    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    // AD CLICK ACTIONS
    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

    /*
     * Method
     */
	function _onLinearAdSkipped(event) {
	    // TO DO
	}

    /*
     * Method
     */
	function _onLinearAdClickThrough(event) {
	    // TO DO
	}

    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    // TRACKING POINT CALLBACKS
    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

    /*
     * Method
     */
	function _onSetTrackingPoint(event) {
	    // TO DO
	}

    /*
     * Method
     */
	function _onTrackingPointFired(event) {
	    // TO DO
	}

    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    // CONTROL BAR CALLBACKS
    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

    /*
     * Method
     */
	function _onControlBarEnable(event) {
	    // TO DO
	}

    /*
     * Method
     */
	function _onControlBarDisable(event) {
	    // TO DO
	}
	
	
    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    // NON-LINEAR ACTIONS
    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

    /*
     * Method
     */
	function _onDisplayOverlay(event) {
	    // TO DO
	}


    /*
     * Method
     */
	function _onHideOverlay(event) {
	    // TO DO
	}


    /*
     * Method
     */
	function _onDisplayNonOverlay(event) {
	    // TO DO
	}


    /*
     * Method
     */
	function _onHideNonOverlay(event) {
	    // TO DO
	}


    /*
     * Method
     */
	function _onOverlayClicked(event) {
	    // TO DO
	}


    /*
     * Method
     */
	function _onOverlayCloseClicked(event) {
	    // TO DO
	}


    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    // COMPANION ACTIONS
    // --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

    /*
     * Method
     */
    this.onDisplayCompanionAd = function() {
	    // TO DO
    }

    /*
     * Method
     */
    this.onHideCompanionAd = function() {
	    // TO DO
    }
}
